/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <ox/mc/read.hpp>
#include <ox/std/std.hpp>

#include "../media.hpp"

#include "addresses.hpp"

namespace nostalgia::core {

ox::Result<char*> loadRom(const char*) noexcept {
	// put the header in the wrong order to prevent mistaking this code for the
	// media section
	constexpr auto headerP2 = "HEADER__________";
	constexpr auto headerP1 = "NOSTALGIA_MEDIA_";
	constexpr auto headerP1Len = ox_strlen(headerP2);
	constexpr auto headerP2Len = ox_strlen(headerP1);
	constexpr auto headerLen = headerP1Len + headerP2Len;
	for (auto current = MEM_ROM; current < reinterpret_cast<char*>(0x0a000000); current += headerLen) {
		if (ox_memcmp(current, headerP1, headerP1Len) == 0 &&
		    ox_memcmp(current + headerP1Len, headerP2, headerP2Len) == 0) {
			return current + headerLen;
		}
	}
	return OxError(1);
}

void unloadRom(char*) noexcept {
}

ox::Result<std::size_t> getPreloadAddr(Context *ctx, const ox::FileAddress &file) noexcept {
	oxRequire(stat, ctx->rom->stat(file));
	oxRequire(buff, ctx->rom->directAccess(file));
	PreloadPtr p;
	oxReturnError(ox::readMC(buff, stat.size, &p));
	return p.preloadAddr + ctx->preloadSectionOffset;
}

}
